@@ -28,6 +28,7 @@ use threads ('yield',
              'stringify');
 use threads::shared;
 my $semaphorus : shared = 0;
+my $command : shared = "sleep";
 my $ZenStyle = 1 ;
 
 # Internationalization
@@ -576,22 +577,31 @@ sub Get {
 	return 0;	
 }	
 
-# Called inside a thread to execute a system command
-# the shared semaphor is set to '1' when system processing ends
+
+# the shared semaphor is set to '0' when system processing ends
 ####################################################################
 # $Netpkg->Execute($command)
+my $thr = threads->new(\&Execute);
+$thr->detach;
+
 sub Execute {
-	my $command = shift;
-	threads->yield();
-	$semaphorus = 0;	
-	`$command`;
-    if ($? == -1) {
-    	$semaphorus = 1;
-		threads->exit(-1); 
-	} else {
-		$semaphorus = 1;
-		threads->exit(0); 
-	} 
+	while (1) {
+		threads->yield();
+		sleep 1;
+		$SIG{'INT'} = sub { threads->exit(0); };
+		$SIG{'TERM'} = sub { threads->exit(0); };
+		$SIG{'KILL'} = sub { threads->exit(0); };
+		$SIG{'QUIT'} = sub { threads->exit(0); };
+		if ($semaphorus == 1) {
+			threads->yield();
+			`$command`;
+			lock $semaphorus;
+			$semaphorus = 0;	
+			lock $command;
+			$command = "sleep";
+			threads->yield();
+		}
+	}
 }
 
 ####################################################################
@@ -599,16 +609,15 @@ sub Execute {
 sub Install {
 	my $self = shift; 
 	my $package = shift;
-	my $command = "/sbin/installpkg $package >/dev/null" ; 
-
+	
     #print "/sbin/installpkg $package\n";
-    
-    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
-    $semaphorus = 0;
-	my $thr = threads->new(\&Execute, $command);
-	while ($semaphorus == 0) {
+       
+	$command = "/sbin/installpkg $package >/dev/null" ; 
+	$semaphorus = 1;
+	until ($semaphorus == 0) {
 		Gtk2->main_iteration while ( Gtk2->events_pending  );  
 	} 
+
 	my $date = strftime('%a %b %d %H:%M:%S %Y',localtime); 
 	open(FH, ">>$self->{Logfile}");
 	print FH "[I] $package $date\n";
@@ -621,13 +630,12 @@ sub Install {
 sub Update {
 	my $self = shift; 
 	my $package = shift;
-	my $command = "/sbin/upgradepkg $package > /dev/null";
+	
     #print "/sbin/upgradepkg --reinstall $package\n";
-    
-    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
-    $semaphorus = 0;
-	my $thr = threads->new(\&Execute, $command);
-	while ($semaphorus == 0) {
+
+	$command = "/sbin/upgradepkg $package > /dev/null";
+	$semaphorus = 1;
+	until ($semaphorus == 0) {
 		Gtk2->main_iteration while ( Gtk2->events_pending  );  
 	}
 	my $date = strftime('%a %b %d %H:%M:%S %Y',localtime); 
@@ -642,13 +650,12 @@ sub Update {
 sub Reinstall {
 	my $self = shift; 
 	my $package = shift;
-	my $command = "/sbin/upgradepkg --reinstall $package >/dev/null";
+	
     #print "/sbin/upgradepkg --reinstall $package\n";
     
-    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
-    $semaphorus = 0;
-	my $thr = threads->new(\&Execute, $command);
-	while ($semaphorus == 0) {
+	$command = "/sbin/upgradepkg --reinstall $package >/dev/null";
+	$semaphorus = 1;
+	until ($semaphorus == 0) {
 		Gtk2->main_iteration while ( Gtk2->events_pending  );  
 	}
 	my $date = strftime('%a %b %d %H:%M:%S %Y',localtime); 
@@ -663,13 +670,11 @@ sub Reinstall {
 sub Remove {
 	my $self = shift; 
 	my $package = shift;
-	my $command = "/sbin/removepkg $package >/dev/null";
     #print "/sbin/removepkg $package\n";
     
-    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
-    $semaphorus = 0;
-	my $thr = threads->new(\&Execute, $command);
-	while ($semaphorus == 0) {
+	$command = "/sbin/removepkg $package >/dev/null";
+	$semaphorus = 1;
+	until ($semaphorus == 0) {
 		Gtk2->main_iteration while ( Gtk2->events_pending  );  
 	}
 	my $date = strftime('%a %b %d %H:%M:%S %Y',localtime); 
@@ -679,6 +684,21 @@ sub Remove {
 	return 0;
 }
 
+####################################################################
+# $Netpkg->AsyncRun( $command );
+sub AsyncRun {
+	my $self = shift; 
+	my $cmd = shift;
+	    
+	$command = "$cmd";
+	$semaphorus = 1;
+	until ($semaphorus == 0) {
+		Gtk2->main_iteration while ( Gtk2->events_pending  );  
+	}
+	return 0;
+}
+
+
 ####################################################################
 # $Netpkg->ProcessDeps(\%db);
 sub ProcessDeps {
@@ -772,20 +792,6 @@ sub SelectDeps {
 	}
 }
 
-####################################################################
-# $Netpkg->AsyncRun( $command );
-sub AsyncRun {
-	my $self = shift; 
-	my $command = shift;
-	    
-    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
-    $semaphorus = 0;
-	my $thr = threads->new(\&Execute, $command);
-	while ($semaphorus == 0) {
-		Gtk2->main_iteration while ( Gtk2->events_pending  );  
-	}
-	return 0;
-}
 
 
 1;
